import "Simbo.Babble.WindowLotro";
import "Simbo.Babble.ConfirmationWindow";
import "Simbo.Babble.SettingsWindow";
import "Simbo.Babble.ColoursWindow";

StatusWindow = class(WindowLotro);

local Constants =
{
	FixedSize =
	{
		Width = 240,
		Height = 340
	},
	Padding =
	{
		Top = 40,
		Left = 20,
		Bottom = 20,
		Right = 20
	},
	Sizes =
	{
		LabelWidth = 110,
		CheckBoxNarrow = 110,
		CheckBoxWide = 190
	},
	LineSpacing = 2,
	ColumnSpacing = 8
}

function StatusWindow:Constructor()
	WindowLotro.Constructor(self);
	
	self.SettingsCheckBox = nil;
	self.Buttons = {};
	
	self:SetText(Strings["babble_header"]);
	self:SetPosition(tonumber(Settings.StatusWindow.Position.Left) or 100, tonumber(Settings.StatusWindow.Position.Top) or 100);
	self:SetSize(Constants.FixedSize.Width, Constants.FixedSize.Height);
	self:SetOpacity(Globals.Opacity);
	
	self.SettingsWindow = SettingsWindow();
	self.SettingsWindow.Saved = function (sender, args)
		local channelType;
		for _, channelType in pairs(ChannelTypes) do
			BabbleHelpers[channelType]:SelectPatternList(Settings.ChannelData[channelType].CurrentPatternList);
			BabbleHelpers[channelType]:SelectNameList(Settings.ChannelData[channelType].CurrentNameList);
		end
		SaveBabbleSettings(false);

		self:EnableChannel(self.SettingsCheckBox, true, false, false);
	end
	
	self.ColoursWindow = ColoursWindow();
	self.ColoursWindow:SetMode(Settings.EnableChatWindow);
	self.ColoursWindow.Saved = function (sender, args)
		SaveBabbleSettings(false);
	end
	
	local top = Constants.Padding.Top;
	local left = Constants.Padding.Left;
	
    self.BabbleEnabledCheckBox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["enable_babble"]);
    self.BabbleEnabledCheckBox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
    self.BabbleEnabledCheckBox.UpdateStatus = true;
	self.BabbleEnabledCheckBox.CheckedChanged = function(sender, args)
		local isChecked = self.BabbleEnabledCheckBox:IsChecked();
		if self.BabbleEnabledCheckBox.UpdateStatus then
			EnableBabble(isChecked);
		else
			self.BabbleEnabledCheckBox.UpdateStatus = true;
		end
		EnableCheckBox(self.ChatWindowCheckbox, isChecked);
		
		for _, channelType in ipairs(SortedChannels) do
			self.Buttons[channelType]:SetEnabled(isChecked);
			EnableCheckBox(self.Buttons[channelType].CheckBox, isChecked and Settings.ChannelData[channelType].CurrentPatternList ~= nil);
		end
	end
	
	top = top + self.BabbleEnabledCheckBox:GetHeight() + Constants.LineSpacing;
	
	self.ChatWindowCheckbox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["babble_chatwindow"]);
    self.ChatWindowCheckbox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ChatWindowCheckbox:SetChecked(Settings.EnableChatWindow);
	self.ChatWindowCheckbox.CheckedChanged = function(sender, args)
		local isChecked = self.ChatWindowCheckbox:IsChecked();
		EnableCheckBox(self.ShowChatWindowCheckbox, isChecked);
		EnableChatWindow(isChecked);
		self.SettingsWindow:EnableStandardButton(isChecked);
		self.ColoursWindow:SetMode(isChecked);
	end
	
	top = top + self.ChatWindowCheckbox:GetHeight();
	left = left + Globals.Sizes.CheckBoxOnly.Width;
	
	self.ShowChatWindowCheckbox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["babble_showchatwindow"]);
    self.ShowChatWindowCheckbox:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.ShowChatWindowCheckbox:SetFont(Globals.Fonts.SmallLabel);
	self.ShowChatWindowCheckbox:SetChecked(Settings.ShowChatWindow);
	self.ShowChatWindowCheckbox.CheckedChanged = function(sender, args)
		if self.ShowChatWindowCheckbox.ManualSet == nil then
			ShowChatWindow(self.ShowChatWindowCheckbox:IsChecked(), false);
		else
			self.ShowChatWindowCheckbox.ManualSet = nil;
		end
	end
	EnableCheckBox(self.ShowChatWindowCheckbox, Settings.EnableChatWindow);
	
	top = top + self.ShowChatWindowCheckbox:GetHeight() + Constants.LineSpacing;
	left = Constants.Padding.Left;
	
	self.EnableStatusButton = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxWide, Globals.Sizes.CheckBoxOnly.Height, Strings["enable_status_button"]);
    self.EnableStatusButton:SetCheckAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	self.EnableStatusButton:SetChecked(Settings.EnableBabbleButton);
	self.EnableStatusButton.CheckedChanged = function(sender, args)
		local isChecked = self.EnableStatusButton:IsChecked();
		StatusButton:SetVisible(self.EnableStatusButton:IsChecked());
		Settings.EnableBabbleButton = isChecked;
	end
	
	top = top + self.EnableStatusButton:GetHeight() + Constants.LineSpacing;
	
	local headerLabel = CreateLabel(self, false);
	headerLabel:SetWidth(Constants.Sizes.LabelWidth);
	headerLabel:SetPosition(left, top);
	headerLabel:SetText(string.upper(Strings["channels"]));
	
	top = top + headerLabel:GetHeight() + Constants.LineSpacing;
	
	for _, channelType in ipairs(SortedChannels) do
		local babbleHelper = BabbleHelpers[channelType];
		local channelData = Settings.ChannelData[channelType];
		local enabled = babbleHelper:GetStatus();
		
		local channelName = Strings["channel_" .. channelType];
		local checkBox = CreateCheckBox(self, left, top, Constants.Sizes.CheckBoxNarrow, Globals.Sizes.CheckBoxOnly.Height, channelName);
		checkBox:SetChecked(enabled);
		checkBox.CheckedChanged = function(sender, args)
			self:EnableChannel(sender, nil, sender:IsChecked(), true);
			if sender:IsChecked() then
				ShowBabbleMessage(Strings["babble_filtering"]:format(channelName));
			else
				ShowBabbleMessage(Strings["babble_not_filtering"]:format(channelName));
			end
		end

		checkBox.ChannelType = channelType;
		self:EnableChannel(checkBox, channelData.CurrentPatternList ~= nil, enabled, false);
		
		left = left + checkBox:GetWidth();
		
		local button = CreateButton(self);
		button:SetPosition(left, top);
		button:SetText(Strings["edit"]);
		button.CheckBox = checkBox;
		button.MouseClick = function(sender, args)
			if args.Button == Turbine.UI.MouseButton.Left then
				if (self.SettingsWindow:IsVisible() and self.SettingsWindow:HasChanges()) or self.SettingsWindow:ChildWindowsOpen() then
					if sender.CheckBox.ChannelType ~= self.SettingsCheckBox.ChannelType then
						self.ConfirmWindow.CheckBox = sender.CheckBox;
						self.ConfirmWindow:ShowWindow(Strings["settings"], Strings["settings_open"], false, true);
					else
						self.SettingsWindow:Activate();
					end
				else
					self.SettingsCheckBox = sender.CheckBox;
					self.SettingsWindow:Show(sender.CheckBox.ChannelType);
				end
			end
		end
		self.Buttons[channelType] = button;
		
		top = top + checkBox:GetHeight() + Constants.LineSpacing;
		left = Constants.Padding.Left;
	end
	
	top = top + Constants.LineSpacing;
	
	local buttonAdjuster = UIOptions["wide_status_button"] and 2 or 1;
	
	self.ColoursButton = CreateButton(self, buttonAdjuster);
	self.ColoursButton:SetPosition((self:GetWidth() - self.ColoursButton:GetWidth()) / 2, top);
	self.ColoursButton:SetText(Strings["edit_colours"]);
	self.ColoursButton.Click = function(sender, args)
		self.ColoursWindow:Show();
	end
	
	self.ConfirmWindow = ConfirmationWindow();
	self.ConfirmWindow:SetZOrder(self:GetZOrder() + 10);
	self.ConfirmWindow.ButtonClick = function (sender,args)
		self.SettingsCheckBox = self.ConfirmWindow.CheckBox;
		self.SettingsWindow:Show(self.ConfirmWindow.CheckBox.ChannelType);
		self.ConfirmWindow.CheckBox = nil;
	end
end

function StatusWindow:Show()
	if not self:IsVisible() then
		ShowChildWindow(self.SettingsWindow);
		ShowChildWindow(self.SettingsWindow.NameListWindow);
		ShowChildWindow(self.SettingsWindow.PatternListWindow);
		ShowChildWindow(self.ColoursWindow);
		self:SetVisible(true);
	end
	self:Activate();
end

function StatusWindow:Hide()
	if self:IsVisible() then
		HideChildWindow(self.SettingsWindow.NameListWindow);
		HideChildWindow(self.SettingsWindow.PatternListWindow);
		HideChildWindow(self.SettingsWindow);
		HideChildWindow(self.ColoursWindow);
		self:SetVisible(false);
	end
end

function StatusWindow:Toggle()
	if self:IsVisible() then
		self:Hide();
	else
		self:Show();
	end
end

function ShowChildWindow(childWindow)
	if childWindow.ReopenOnToggle then
		childWindow.ReopenOnToggle = false;
		childWindow:SetVisible(true);
	end
end

function HideChildWindow(childWindow)
	if childWindow:IsVisible() then
		childWindow.ReopenOnToggle = true;
		childWindow:SetVisible(false);
	end
end

function StatusWindow:EnableChannel(checkBox, enableCheckBox, channelEnabled, updateHelper)
	local label = checkBox.Label;
	EnableCheckBox(checkBox, enableCheckBox);
	
	if updateHelper then
		local channelType = checkBox.ChannelType;
		local babbleHelper = BabbleHelpers[channelType];
		Settings.ChannelData[channelType].Enabled = channelEnabled;
		babbleHelper:UpdateStatus(channelEnabled);
	end
end

function StatusWindow:EnableBabbleCheckBox(enabled)
	self.BabbleEnabledCheckBox.UpdateStatus = false;
	self.BabbleEnabledCheckBox:SetChecked(enabled);
end

function StatusWindow:SavePositionSettings()
	Settings.StatusWindow.Position.Top = tostring(self:GetTop());
	Settings.StatusWindow.Position.Left = tostring(self:GetLeft());
	
	self.SettingsWindow:SavePositionSettings();
	self.ColoursWindow:SavePositionSettings();
end